@@ -55,15 +55,13 @@
      tROOT4 = 271,
      tROOT3 = 272,
      tROOT = 273,
-     tFUNCTION = 274,
-     tPI = 275,
-     tVARIABLE = 276,
-     tSUPNUM = 277,
-     tSUBNUM = 278,
-     BOOLEAN_OPERATOR = 279,
-     PERCENTAGE = 280,
-     UNARY_MINUS = 281,
-     tINVERSE = 282
+     tVARIABLE = 274,
+     tSUPNUM = 275,
+     tSUBNUM = 276,
+     BOOLEAN_OPERATOR = 277,
+     PERCENTAGE = 278,
+     UNARY_MINUS = 279,
+     tINVERSE = 280
    };
 #endif
 /* Tokens.  */
@@ -83,15 +81,13 @@
 #define tROOT4 271
 #define tROOT3 272
 #define tROOT 273
-#define tFUNCTION 274
-#define tPI 275
-#define tVARIABLE 276
-#define tSUPNUM 277
-#define tSUBNUM 278
-#define BOOLEAN_OPERATOR 279
-#define PERCENTAGE 280
-#define UNARY_MINUS 281
-#define tINVERSE 282
+#define tVARIABLE 274
+#define tSUPNUM 275
+#define tSUBNUM 276
+#define BOOLEAN_OPERATOR 277
+#define PERCENTAGE 278
+#define UNARY_MINUS 279
+#define tINVERSE 280
 
 
 
@@ -101,7 +97,7 @@ typedef union YYSTYPE
 {
 
 /* Line 1676 of yacc.c  */
-#line 101 "./mp-equation-parser.y"
+#line 81 "./mp-equation-parser.y"
 
   MPNumber int_t;
   int integer;
@@ -110,7 +106,7 @@ typedef union YYSTYPE
 
 
 /* Line 1676 of yacc.c  */
-#line 114 "mp-equation-parser.h"
+#line 110 "mp-equation-parser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
