@@ -1,7 +1,4 @@
-
-/*  $Header$
- *
- *  Copyright (C) 2004-2008 Sami Pietila
+/*  Copyright (c) 2004-2008 Sami Pietila
  *  Copyright (c) 2008-2009 Robert Ancell
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,47 +22,43 @@
 
 #include "mp.h"
 
-#define PARSER_ERR_INVALID          1
-#define PARSER_ERR_TOO_LONG_NUMBER  2
-#define PARSER_ERR_BITWISEOP        3
-#define PARSER_ERR_MODULUSOP        4
-#define PARSER_ERR_OVERFLOW         5
-#define PARSER_ERR_UNKNOWN_VARIABLE 6
-#define PARSER_ERR_UNKNOWN_FUNCTION 7
-#define PARSER_ERR_INVALID_BASE     8
-
-typedef struct MPEquationParserState MPEquationParserState;
-
-/* State for parser */
-struct MPEquationParserState {
-    /* The numeric base (e.g 2, 8, 10, 16) */
-    int base;
+typedef enum
+{
+    PARSER_ERR_NONE = 0,
+    PARSER_ERR_INVALID,
+    PARSER_ERR_OVERFLOW,
+    PARSER_ERR_UNKNOWN_VARIABLE,
+    PARSER_ERR_UNKNOWN_FUNCTION,
+    PARSER_ERR_MP
+} MPErrorCode;
 
+/* Options for parser */
+typedef struct {
     /* The wordlength for binary operations in bits (e.g. 8, 16, 32) */
     int wordlen;
-    
+
     /* Units for angles (e.g. radians, degrees) */
     MPAngleUnit angle_units;
-    
+
+    // FIXME:
+    // int enable_builtins;
+
+    /* Data to pass to callbacks */
+    void *callback_data;
+
     /* Function to get variable values */
-    int (*get_variable)(MPEquationParserState *state, const char *name, MPNumber *z);
-    
+    int (*get_variable)(const char *name, MPNumber *z, void *data);
+
     /* Function to set variable values */
-    void (*set_variable)(MPEquationParserState *state, const char *name, MPNumber *x);    
+    void (*set_variable)(const char *name, const MPNumber *x, void *data);
 
     /* Function to solve functions */
-    int (*get_function)(MPEquationParserState *state, const char *name, const MPNumber *x, MPNumber *z);
-    
-    // FIXME: get_operator??
+    int (*get_function)(const char *name, const MPNumber *x, MPNumber *z, void *data);
+} MPEquationOptions;
 
-    /* Error returned from parser */
-    int error;
-
-    /* Value returned from parser */
-    MPNumber ret;
-};
-
-int mp_equation_parse(const char *expression, MPNumber *result);
-int _mp_equation_error(void *yylloc, MPEquationParserState *state, char *text);
+MPErrorCode mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *result, char **error_token);
+const char *mp_error_code_to_string(MPErrorCode error_code);
 
+int sub_atoi(const char *data);
+int super_atoi(const char *data);
 #endif
